==> nui_core-0.0.83.tar.gz: lib/ui/component/nui_zoom_overlay.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:vector_math/vector_math_64.dart';

class NUITransformWidget extends StatefulWidget {
  final Widget child;
  final Matrix4 matrix;

  const NUITransformWidget({Key key, @required this.child, @required this.matrix})
      : assert(child != null),
        super(key: key);

  @override
  _NUITransformWidgetState createState() => _NUITransformWidgetState();
}

class _NUITransformWidgetState extends State<NUITransformWidget> {
  Matrix4 _matrix = Matrix4.identity();

  @override
  Widget build(BuildContext context) {
    return _matrix != null
        ? Transform(
      transform: (widget.matrix * _matrix),
      child: widget.child,
    )
        : Container();
  }

  void setMatrix(Matrix4 matrix) {
    setState(() {
      _matrix = matrix;
    });
  }
}

//
// ZoomOverlay enables a image to have full screen drag, pinch and zoom
//
class NUIZoomOverlay extends StatefulWidget {
  final Widget child;
  final bool twoTouchOnly;

  const NUIZoomOverlay({
    Key key,
    @required this.twoTouchOnly,
    @required this.child,
  })  : assert(child != null),
        super(key: key);

  @override
  _NUIZoomOverlayState createState() => _NUIZoomOverlayState();
}

class _NUIZoomOverlayState extends State<NUIZoomOverlay>
    with TickerProviderStateMixin {
  Matrix4 _matrix = Matrix4.identity();
  Offset _startFocalPoint;
  Animation<Matrix4> _animationReset;
  AnimationController _controllerReset;
  OverlayEntry _overlayEntry;
  bool _isZooming = false;
  int _touchCount = 0;
  Matrix4 _transformMatrix = Matrix4.identity();

  final GlobalKey<_NUITransformWidgetState> _transformWidget =
  GlobalKey<_NUITransformWidgetState>();

  @override
  void initState() {
    super.initState();

    _controllerReset =
        AnimationController(vsync: this, duration: Duration(milliseconds: 100));

    _controllerReset.addListener(() {
      _transformWidget.currentState.setMatrix(_animationReset.value);
    });

    _controllerReset.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        hide();
      }
    });
  }

  @override
  void dispose() {
    _controllerReset.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Listener(
        onPointerDown: _incrementEnter,
        onPointerUp: _incrementExit,
        child: GestureDetector(
            onScaleStart: (details) {
              onScaleStart(details, context);
            },
            onScaleUpdate: onScaleUpdate,
            onScaleEnd: onScaleEnd,
            child: Opacity(opacity: _isZooming ? 0 : 1, child: widget.child)));
  }

  void onScaleStart(ScaleStartDetails details, BuildContext context) {
    if (widget.twoTouchOnly && _touchCount < 2) return;
    _startFocalPoint = details.focalPoint;
    _matrix = Matrix4.identity();

    // create an matrix of where the image is on the screen for the overlay
    RenderBox renderBox = context.findRenderObject();
    Offset position = renderBox.localToGlobal(Offset.zero);
    _transformMatrix =
        Matrix4.translation(Vector3(position.dx, position.dy, 0));

    show(context);

    setState(() {
      _isZooming = true;
    });
  }

  void onScaleUpdate(ScaleUpdateDetails details) {
    if (!_isZooming) return;

    Offset translationDelta = details.focalPoint - _startFocalPoint;

    Matrix4 translate = Matrix4.translation(
        Vector3(translationDelta.dx, translationDelta.dy, 0));

    RenderBox renderBox = context.findRenderObject();
    Offset focalPoint =
    renderBox.globalToLocal(details.focalPoint - translationDelta);

    var dx = (1 - details.scale) * focalPoint.dx;
    var dy = (1 - details.scale) * focalPoint.dy;

    Matrix4 scale = Matrix4(details.scale, 0, 0, 0, 0, details.scale, 0, 0, 0,
        0, 1, 0, dx, dy, 0, 1);

    _matrix = translate * scale;

    if (_transformWidget != null && _transformWidget.currentState != null) {
      _transformWidget.currentState.setMatrix(_matrix);
    }
  }

  void onScaleEnd(ScaleEndDetails details) {
    if (!_isZooming) return;

    _animationReset = Matrix4Tween(begin: _matrix, end: Matrix4.identity())
        .animate(_controllerReset);
    _controllerReset.reset();
    _controllerReset.forward();
  }

  Widget _build(BuildContext context) {
    return IgnorePointer(
      child: Stack(
        children: [
          NUITransformWidget(
            key: _transformWidget,
            matrix: _transformMatrix,
            child: widget.child,
          )
        ],
      ),
    );
  }

  void show(BuildContext context) async {
    if (!_isZooming) {
      OverlayState overlayState = Overlay.of(context);
      _overlayEntry = new OverlayEntry(builder: _build);
      overlayState.insert(_overlayEntry);
    }
  }

  void hide() async {
    setState(() {
      _isZooming = false;
    });

    _overlayEntry.remove();
    _overlayEntry = null;
  }

  void _incrementEnter(PointerEvent details) {
    _touchCount++;
  }

  void _incrementExit(PointerEvent details) {
    _touchCount--;
  }
}

==> nui_core-0.0.84.tar.gz: lib/ui/component/nui_zoom_overlay.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:nui_core/nui_core.dart';
import 'package:vector_math/vector_math_64.dart';

class _NUITransformWidget extends StatefulWidget {
  final Widget child;
  final Matrix4 matrix;

  const _NUITransformWidget({Key key, @required this.child, @required this.matrix})
      : assert(child != null),
        super(key: key);

  @override
  _NUITransformWidgetState createState() => _NUITransformWidgetState();
}

class _NUITransformWidgetState extends State<_NUITransformWidget> {
  Matrix4 _matrix = Matrix4.identity();

  @override
  Widget build(BuildContext context) {
    return _matrix != null
        ? Transform(
      transform: (widget.matrix * _matrix),
      child: widget.child,
    ) : Container();
  }

  void setMatrix(Matrix4 matrix) {
    setState(() {
      _matrix = matrix;
    });
  }
}

//
// ZoomOverlay enables a image to have full screen drag, pinch and zoom
//
class NUIZoomOverlay extends StatefulWidget {
  final Widget child;
  final bool twoTouchOnly;

  const NUIZoomOverlay({
    Key key,
    @required this.twoTouchOnly,
    @required this.child,
  })  : assert(child != null),
        super(key: key);

  @override
  _NUIZoomOverlayState createState() => _NUIZoomOverlayState();
}

class _NUIZoomOverlayState extends State<NUIZoomOverlay>
    with TickerProviderStateMixin {
  Matrix4 _matrix = Matrix4.identity();
  Offset _startFocalPoint;
  Animation<Matrix4> _animationReset;
  AnimationController _controllerReset;
  OverlayEntry _overlayEntry;
  bool _isZooming = false;
  int _touchCount = 0;
  Matrix4 _transformMatrix = Matrix4.identity();
  double _screenWidth;

  final GlobalKey<_NUITransformWidgetState> _transformWidget =
  GlobalKey<_NUITransformWidgetState>();

  @override
  void initState() {
    super.initState();

    _controllerReset = AnimationController(vsync: this, duration: Duration(milliseconds: 150));

    _controllerReset.addListener(() {
      _transformWidget.currentState.setMatrix(_animationReset.value);
    });

    _controllerReset.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        hide();
      }
    });
  }

  @override
  void dispose() {
    _controllerReset.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _screenWidth = _screenWidth ?? screenWidth(context);
    return Listener(
        onPointerDown: _incrementEnter,
        onPointerUp: _incrementExit,
        child: GestureDetector(
            onScaleStart: (details) {
              onScaleStart(details, context);
            },
            onScaleUpdate: onScaleUpdate,
            onScaleEnd: onScaleEnd,
            child: Opacity(opacity: _isZooming ? 0 : 1, child: widget.child)));
  }

  void onScaleStart(ScaleStartDetails details, BuildContext context) {
    if (widget.twoTouchOnly && _touchCount < 2) return;
    _startFocalPoint = details.focalPoint;
    _matrix = Matrix4.identity();

    // create an matrix of where the image is on the screen for the overlay
    RenderBox renderBox = context.findRenderObject();

    final diffWidthRatio = (renderBox.size.width/_screenWidth);
    Offset position = renderBox.localToGlobal(Offset.zero);

    _transformMatrix = Matrix4(
        1 * diffWidthRatio, 0, 0, 0,
        0,  1 * diffWidthRatio, 0, 0,
        0, 0, 1, 0,
        position.dx, position.dy, 0, 1
    );

    show(context);

    setState(() {
      _isZooming = true;
    });
  }

  void onScaleUpdate(ScaleUpdateDetails details) {
    if (!_isZooming) return;

    Offset translationDelta = details.focalPoint - _startFocalPoint;

    Matrix4 translate = Matrix4.translation(Vector3(translationDelta.dx, translationDelta.dy, 0));

    RenderBox renderBox = context.findRenderObject();
    Offset focalPoint = renderBox.globalToLocal(details.focalPoint - translationDelta);

    var dx = (1 - details.scale) * focalPoint.dx;
    var dy = (1 - details.scale) * focalPoint.dy;

    Matrix4 scale = Matrix4(
        details.scale, 0, 0, 0,
        0, details.scale, 0, 0,
        0, 0, 1, 0,
        dx, dy, 0, 1);

    _matrix = translate * scale;

    if (_transformWidget != null && _transformWidget.currentState != null) {
      _transformWidget.currentState.setMatrix(_matrix);
    }
  }

  void onScaleEnd(ScaleEndDetails details) {
    if (!_isZooming) return;

    _animationReset = Matrix4Tween(begin: _matrix, end: Matrix4.identity()).animate(_controllerReset);
    _controllerReset.reset();
    _controllerReset.forward();
  }

  Widget _build(BuildContext context) {
    return IgnorePointer(
      child: Stack(
        children: [
          _NUITransformWidget(
            key: _transformWidget,
            matrix: _transformMatrix,
            child: widget.child,
          )
        ],
      ),
    );
  }

  void show(BuildContext context) async {
    if (!_isZooming) {
      OverlayState overlayState = Overlay.of(context);
      _overlayEntry = new OverlayEntry(builder: _build);
      overlayState.insert(_overlayEntry);
    }
  }

  void hide() async {
    setState(() {
      _isZooming = false;
    });

    _overlayEntry.remove();
    _overlayEntry = null;
  }

  void _incrementEnter(PointerEvent details) {
    _touchCount++;
  }

  void _incrementExit(PointerEvent details) {
    _touchCount--;
  }
}
